
/*
  Copyright 1995 The University of Rhode Island and The Massachusetts
  Institute of Technology

  Portions of this software were developed by the Graduate School of
  Oceanography (GSO) at the University of Rhode Island (URI) in collaboration
  with The Massachusetts Institute of Technology (MIT).

  Access and use of this software shall impose the following obligations and
  understandings on the user. The user is granted the right, without any fee
  or cost, to use, copy, modify, alter, enhance and distribute this software,
  and any derivative works thereof, and its supporting documentation for any
  purpose whatsoever, provided that this entire notice appears in all copies
  of the software, derivative works and supporting documentation.  Further,
  the user agrees to credit URI/MIT in any publications that result from the
  use of this software or in any product that includes this software. The
  names URI, MIT and/or GSO, however, may not be used in any advertising or
  publicity to endorse or promote any products or commercial entity unless
  specific written permission is obtained from URI/MIT. The user also
  understands that URI/MIT is not obligated to provide the user with any
  support, consulting, training or assistance of any kind with regard to the
  use, operation and performance of this software nor to provide the user
  with any updates, revisions, new versions or "bug fixes".

  THIS SOFTWARE IS PROVIDED BY URI/MIT "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL URI/MIT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS
  ACTION, ARISING OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE
  OF THIS SOFTWARE.
*/

//
// DODS-netCDF surrogate library for cdf file inquire. 
// 
// ReZa 10/13/94

// $Log: Dcdf.cc,v $
// Revision 1.6  1996/09/17 17:06:08  jimg
// Merge the release-2-0 tagged files (which were off on a branch) back into
// the trunk revision.
//
// Revision 1.5.4.3  1996/09/17 00:38:32  jimg
// Fixed broken log comments.
//
// Revision 1.5.4.2  1996/09/17 00:26:05  jimg
// Merged changes from a side branch which contained various changes from
// Reza and Charles.
// Removed ncdump and netexec since ncdump is now in its own directory and
// netexec is no longer used.
//
// Revision 1.5.4.1  1996/06/25 22:04:06  jimg
// Version 2.0 from Reza.
//
// Revision 1.5  1995/07/09  21:33:30  jimg
// Added copyright notice.
//
// Revision 1.4  1994/12/22  04:51:43  reza
// Updated to use DODS new named dimension capability.
//
// Revision 1.3  1994/12/08  19:01:32  reza
// Added support for local netCDF file access through the original netCDF code.
//
// Revision 1.2  1994/11/23  21:05:53  reza
// First working version.
//
// Revision 1.1  1994/11/03  04:37:57  reza
// Preliminary version of netCDF -> DODS library.

#include "config_nc.h"

static char rcsid[] __unused__ ={"$Id: Dcdf.cc,v 1.6 1996/09/17 17:06:08 jimg Exp $"};

#include <assert.h>
#include "Dnetcdf.h"

int
ncinquire(int cdfid,int *ndimsp,int *nvarsp, int *nattrsp, int *xtendimp)
{
if((*conns)[cdfid] == NULL) // does the file id exist?
    return(-1) ;

#ifdef LOCAL
  if((*conns)[cdfid]->is_local()) {
    rcode = lncinquire((*conns)[cdfid]->ncid(), ndimsp, nvarsp,
			      nattrsp,xtendimp);
    return rcode;
  }
#endif
  
  cdf_routine_name = "ncinquire" ;

  if(nvarsp != NULL)
    *nvarsp = (*conns)[cdfid]->nvars();

  extern int num_attr(int cdfid, int varid);
  if(nattrsp != NULL)
    *nattrsp = num_attr(cdfid,NC_GLOBAL);

  if(ndimsp != NULL)
    *ndimsp = (*conns)[cdfid]->ndims();
		
  if(xtendimp != NULL)
    *xtendimp = (*conns)[cdfid]->recdim(); 
	
  return(cdfid) ;
}
