/*
  Copyright 1995 The University of Rhode Island and The Massachusetts
  Institute of Technology

  Portions of this software were developed by the Graduate School of
  Oceanography (GSO) at the University of Rhode Island (URI) in collaboration
  with The Massachusetts Institute of Technology (MIT).

  Access and use of this software shall impose the following obligations and
  understandings on the user. The user is granted the right, without any fee
  or cost, to use, copy, modify, alter, enhance and distribute this software,
  and any derivative works thereof, and its supporting documentation for any
  purpose whatsoever, provided that this entire notice appears in all copies
  of the software, derivative works and supporting documentation.  Further,
  the user agrees to credit URI/MIT in any publications that result from the
  use of this software or in any product that includes this software. The
  names URI, MIT and/or GSO, however, may not be used in any advertising or
  publicity to endorse or promote any products or commercial entity unless
  specific written permission is obtained from URI/MIT. The user also
  understands that URI/MIT is not obligated to provide the user with any
  support, consulting, training or assistance of any kind with regard to the
  use, operation and performance of this software nor to provide the user
  with any updates, revisions, new versions or "bug fixes".

  THIS SOFTWARE IS PROVIDED BY URI/MIT "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL URI/MIT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS
  ACTION, ARISING OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE
  OF THIS SOFTWARE.
*/

//
// DODS-netCDF surrogate library for data retrieval. 
// 
// ReZa 10/14/94

// $Log: Dputget.cc,v $
// Revision 1.18  1998/02/05 20:14:44  jimg
// DODS now compiles with gcc 2.8.x
//
// Revision 1.17  1997/03/04 17:18:18  jimg
// Added `delete Rdds' to DODvario so that DDS objects allocated on the heap
// are freed once they are no longer needed.
// Changed the call `free tmpbufout' to `delete tmpbufout' since tmpbufout is
// allocated using new (see type_conv).
//
// Revision 1.16  1997/02/12 06:37:47  reza
// Changed DDS returned by request_data to a pointer.
// Add check for Null DDS and turned the GUI on.
//
// Revision 1.15  1996/09/17 17:06:09  jimg
// Merge the release-2-0 tagged files (which were off on a branch) back into
// the trunk revision.
//
// Revision 1.14.2.4  1996/09/17 00:26:10  jimg
// Merged changes from a side branch which contained various changes from
// Reza and Charles.
// Removed ncdump and netexec since ncdump is now in its own directory and
// netexec is no longer used.
//
// Revision 1.14.2.3  1996/08/13 21:18:57  jimg
// *** empty log message ***
//
// Revision 1.14.2.2  1996/07/10 21:43:49  jimg
// Changes for version 2.06. These fixed lingering problems from the migration
// from version 1.x to version 2.x.
// Removed some (but not all) warning generated with gcc's -Wall option.
//
// Revision 1.14.2.1  1996/06/25 22:04:12  jimg
// Version 2.0 from Reza.
//
// Revision 1.14  1995/11/18  11:33:22  reza
// Updated member function names for DAP-1.1.1.
//
// Revision 1.13  1995/07/09  21:33:34  jimg
// Added copyright notice.
//
// Revision 1.12  1995/06/28  20:22:36  jimg
// Replaced malloc calls with calls to new (and calls to free with calls to
// delete).
//
// Revision 1.11  1995/06/23  20:14:08  reza
// Updated DODvario() call.
//
// Revision 1.10  1995/06/23  19:53:32  jimg
// Added break.
// Reformatted.
//
// Revision 1.9  1995/06/23  15:27:00  jimg
// Changed DODvario() so that it makes use of the new netio Connect class
// methods.
//
// Revision 1.8  1995/04/28  20:39:57  reza
// Moved hyperslab access to the server side (passed as a constraint argument).
//
// Revision 1.7  1995/03/23  15:03:09  reza
// Fixed a bug in HyperGet recursion.
//
// Revision 1.6  1995/03/16  16:56:24  reza
// Updated for the new DAP. All the read_val mfunc. and their memory management
// are now moved to their parent class.
// Data transfers are now in binary while DAS and DDS are still sent in ASCII.
//
// Revision 1.5  1995/02/10  04:51:58  reza
// Finished data transmiton code and added Hyperslab and type conversion func.
//
// Revision 1.4  1994/12/22  04:45:34  reza
// Rearranged for simpler DODS data transmission code interface.
//
// Revision 1.3  1994/12/08  19:01:30  reza
// Added support for local netCDF file access through the original netCDF code.
//
// Revision 1.2  1994/11/23  21:06:00  reza
// First working version.
//
// Revision 1.1  1994/11/03  04:35:47  reza
// Preliminary version of netCDF -> DODS library.

#include "config_nc.h"

static char rcsid[] __unused__ ={"$Id: Dputget.cc,v 1.18 1998/02/05 20:14:44 jimg Exp $"};

#include <assert.h>
#include "Dnetcdf.h"

extern var_info(int cdfid,Pix p, nc_type *typep,int *ndimsp, int dims[]);

//
// Function to get a hyperslap of the data into the requested buffer.
//
int
HyperGet(const int dim_num, int indx, const int dimsz[], nc_type typep, 
	 const long *start, const long *edge, void *buff, void *value)
{
    long jump = 1;
    int szof = nctypelen(typep);

    if(dim_num != (indx+1)){
	// number of lines, pages, etc to skip 
	for (int i = dim_num-1; i > indx; i--)
	    jump *= dimsz[i];

	for (int edg = 0; edg < edge[indx] ; edg++){
	    void *bufft = ((char *)buff + (start[indx] + edg) * jump * szof);
	    value = (char *)value + HyperGet(dim_num, indx+1, dimsz, typep, start, edge, bufft, value);
	}
	return (0);
    }
    else {
	//number of bytes to jump
	void *bufft = (char *)buff + start[indx] * szof ;
	memcpy((void *) value, (void *) bufft, (size_t)edge[indx]*szof);
	return (edge[indx] * szof);
    }
}

// type conversion from DODS to the local machine

void *	  
type_conv(nc_type datatype, int nels, void *tmpbufin)
{
    dods_float64 *flt64;
    dods_int32 *intg32;
    String *strg;
    int id;
    void *tmpbufout = tmpbufin;

    if (datatype == NC_CHAR) {
	char *chr = new char [nels * nctypelen(datatype)];
	strg = (String *) tmpbufin;
	tmpbufout = chr;

	for (id = 0; id < nels; id++) 
	    *chr++ = *((const char *)*(strg++));
    }

    if ((datatype == NC_DOUBLE) 
	&& (sizeof(dods_float64) != lnctypelen(datatype))) {
	double *dbl = (double *)new char [(nels*nctypelen(datatype))];

	flt64 = (dods_float64 *)tmpbufin;
	tmpbufout = dbl;

	for (id = 0; id < nels; id++)
	    *dbl++ = (double) *flt64++;
    }

    if ((datatype == NC_LONG) 
	&& (sizeof(dods_int32) != lnctypelen(datatype))) {
	nclong *lng = (nclong *)new char [(nels*nctypelen(datatype))];
	intg32 = (dods_int32 *)tmpbufin;
	tmpbufout = lng;

	for (id = 0; id < nels; id++) 
	    *lng++ = (nclong) *intg32++;
    }


#ifdef NEVER
// Curently there is no need for types FLOAT and SHORT since all of the
// netCDF data types are mapped into DOUBLE and LONG variables.
// Done for possible future changes..


    if ((datatype == NC_FLOAT) && (sizeof(float64) != lnctypelen(datatype))){
	float *flt = (float *)new char [nels*nctypelen(datatype)];
	flt64 = (float64 *)tmpbufin;
	tmpbufout = flt;

	for (id = 0; id < nels; id++) 
	    *flt++ = (float) *flt64++;
    }


    if ((datatype == NC_SHORT) && (sizeof(int32) != lnctypelen(datatype))){
	short *sht = (short *)new char [nels*nctypelen(datatype)];
	intg32 = (int32 *)tmpbufin;
	tmpbufout = sht;

	for (id = 0; id < nels; id++) 
	    *sht++ = (short) *intg32++;      
    }
#endif

    return tmpbufout;
}

int
DODvario(int cdfid, int varid, const long *start, const long *edges,
	 const long *stride, void *values)
{
    static char start_rep[MAX_NC_NAME];
    static char step_rep[MAX_NC_NAME];
    static char stop_rep[MAX_NC_NAME];
    nc_type typep;
    int ndimsp, nels = 1;
    int dims[MAX_NC_DIMS];

#ifdef OldServer
    int dimsz[MAX_NC_DIMS];
#endif

    NCConnect *c = (*conns)[cdfid];
    if(!c)			// does the file id exist?
	return(-1);

    //select the Pix for the given varid
    Pix p = c->Cdds().first_var();
    for (int i=0; i < varid ; i++)
	c->Cdds().next_var(p);

    String var_name = c->Cdds().var(p)->name();

    // Collect info about the variable
    var_info(cdfid,p,&typep,&ndimsp,dims);

    // Get dimension sizes and strings for constraint hyperslab
    String expr = var_name;
    int dm = 0;
    int ext_start, ext_step, ext_stop; //external constraints
    
    // Read the Array or the Array part of the Grid
    Array *ar = 0;
    bool found_var = false;
    for(Pix Cp=c->dds().first_var(); Cp; c->dds().next_var(Cp)) 
	if(c->dds().var(Cp)->name() == var_name) { 
	    switch (c->dds().var(Cp)->type()){
	      case dods_grid_c: {
		  Grid *gr = (Grid *)c->dds().var(Cp); 
		  ar =  (Array *)gr->array_var();
		  found_var = true;
		  break;
	      }
	      case dods_array_c: {
		  ar = (Array *)c->dds().var(Cp); 
		  found_var = true;
		  break;		
	      }
	      default:
		cerr << " Wrong variable type " <<endl;
		return (-1); // wrong type
	    }
	    break;
	}

    if(!found_var) {
	cerr << " Could not find variable: " << var_name <<endl;
	return (-1); // no such a variable
    }
    
    for (Pix d = ar->first_dim(); d; ar->next_dim(d), dm++) {

	//Verify stride argument.
	if (stride != NULL && stride[dm] < 1) {
	    NCadvise(NC_EINVAL, "Non-positive stride");
	    return(-1) ;
	}
   
	// Set defaults:
	// *start         NULL => first corner 
	// *edges         NULL => everything following start[] 
	// *stride        NULL => unity strides 
	long instart = start != NULL
	    ? start[dm]
	    : 0;
	long inedges = edges != NULL
	    ? edges[dm]
	    : dims[dm] - instart;
	long instep = stride != NULL
	    ? stride[dm]
	    : 1;

	// external constarint (from ncopen)
	ext_start = ar->dimension_start(d, true); 
	ext_step = ar->dimension_stride(d, true);
	ext_stop = ar->dimension_stop(d, true);

#ifdef TEST      
	cout << instart <<" "<< inedges <<" "<< dm << endl;
	cout << ext_start <<" "<< ext_step <<" "<<ext_stop<<endl;
#endif

	// create constarint expr. by combining the constarints
	int Tstart = ext_start+instart*ext_step;
	int Tstep = instep*ext_step;
	int Tstop = 
	    (ext_stop < ext_start+(instart+(inedges-1)*instep)*ext_step) 
	    ? ext_stop : ext_start+(instart+(inedges-1)*instep)*ext_step;
      
	sprintf(start_rep,"%d",Tstart);
	sprintf(step_rep,"%d",Tstep);	      
	sprintf(stop_rep,"%d",Tstop);
      
	expr = expr + "[" + start_rep + ":" + step_rep + ":" + stop_rep + "]";

	nels *= inedges;

    }

#ifdef TEST
    cout << expr <<endl;
#endif
    
    // ask for the sync. data with GUI on...

    DDS *Rdds = c->request_data(expr, true, false); 

    //NULL return, no data was sent from the server.
    if (!Rdds) {
	cerr << "Error reading data" << endl;
	return (-1);
    }
  
    // By definition, the DDS `Rdds' can have only one variable, so make 
    // sure that is true.
    if (Rdds->num_var() != 1) { 
	cerr << "Too many variables; use only single variable projections"
	     << endl;
	return (-1);
    }

    // moved to Connect.cc  if ( c->Cdds().var(p)->deserialize(c->source()) ){
      
    void *tmpbufin = 0;

    Pix qb = Rdds->first_var();  // Only one variable in this dds
    switch (Rdds->var(qb)->type()){
      case dods_grid_c: {
	  Grid *gr = (Grid *)Rdds->var(qb); 
	  ar =  (Array *)gr->array_var();
	  break;
      }
      case dods_array_c: {
	  ar = (Array *)Rdds->var(qb); 
	  break;		
      }
      default:
	cerr << " Wrong variable type " <<endl;
	return (-1); // wrong type
    }

    assert(ar);		// make sure AR is not null

    rcode =  ar->buf2val((void **)&tmpbufin); 

    if ( !rcode ){
	cerr << "Unable to retrieve value" << endl;
	delete Rdds;
	return (-1);
    }

    //conver to local variable types

    void *tmpbufout =  type_conv(typep, nels, tmpbufin);
    int szof = nctypelen(typep);
    memcpy((void *) values, (void *) tmpbufout, (size_t)nels*szof);

#ifdef OldServer
    // get the part asked for
    (void) HyperGet(ndimsp,0,dimsz,typep,start,edges,
		    (void *)tmpbufout,values);
#endif 

    if(tmpbufin != tmpbufout)
	delete(tmpbufout); 

    delete Rdds;

    return (0) ;
}

int
ncvarget1(int cdfid, int varid, const long *coords, ncvoid *value)
{
    static long count[MAX_NC_DIMS];

    if((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncvarget1((*conns)[cdfid]->ncid(),varid, coords, value);
	return rcode;
    }
#endif
    cdf_routine_name = "ncvarget1" ;
    for (int i =0; i < MAX_NC_DIMS ; i++)
	count[i] = 1;     // set count to one

    return DODvario(cdfid, varid, coords, count, 0, value);
}


int
ncvarget(int cdfid, int varid, const long *start, 
	 const long *edges, ncvoid *values)
{
    if((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncvarget((*conns)[cdfid]->ncid(),varid, start, edges, values);
	return rcode;
    }
#endif
    cdf_routine_name = "ncvarget" ;

    return DODvario(cdfid, varid, start, edges, 0, values);
}


int
ncrecget(int cdfid, long recnum, ncvoid **datap)
{
    if((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncrecget((*conns)[cdfid]->ncid(),recnum, datap);
	return rcode;
    }
#endif
    cdf_routine_name = "ncrecget" ;

    NCadvise(NC_EPERM, "Record access is not supported yet, DODS") ;
    return(-1);

}


int
ncrecinq(int cdfid,int *nrecvars, int *recvarids, long *recsizes)
{
    if((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode=lncrecinq((*conns)[cdfid]->ncid(),nrecvars,recvarids,recsizes);
	return rcode;
    }
#endif
    cdf_routine_name = "ncrecinq" ;
	
    int nrvars = -1; // <-----get number of variables with unlimited dimension
    if(nrvars == -1)
	return(-1) ;

    if(nrecvars != NULL)
	*nrecvars = nrvars;
	
    if(recvarids != NULL) {
	int ii ;
	for(ii = 0 ; ii < nrvars ; ii++) {
	    recvarids[ii] = 0; // <-get variable ID for each unlimited variable
	}
    }

    if(recsizes != NULL) {
	int ii ;
	for(ii = 0 ; ii < nrvars ; ii++) {
	    recsizes[ii] = 0; // <--get record size for each UL variable ---
	}
    }

    return nrvars ;
}


int
ncvarput1(int cdfid, int varid, const long *coords, const ncvoid *value)
{
    if((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncvarput1((*conns)[cdfid]->ncid(), varid, coords, value);
	return rcode;
    }
#endif
    cdf_routine_name = "ncvarput1" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}


int
ncvarput(int cdfid, int varid, const long *start, 
	 const long *edges, const ncvoid *values)
{
    if((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncvarput((*conns)[cdfid]->ncid(),varid,start,edges,values);
	return rcode;
    }
#endif
    cdf_routine_name = "ncvarput" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}


int
ncrecput(int cdfid, long recnum, ncvoid * const *datap)
{
    if((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncrecput((*conns)[cdfid]->ncid(), recnum, datap);
	return rcode;
    }
#endif
    cdf_routine_name = "ncrecput" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}

/* 
 * external interface function
 *  this is how much space is required by the user, as in
 *
 *   vals = malloc(nel * nctypelen(var.type));
 *   ncvarget (cdfid, varid, cor, edg, vals);
 *
 */

int
nctypelen(nc_type type)
{
    return lnctypelen(type);
}
