/*
  Copyright 1995 The University of Rhode Island and The Massachusetts
  Institute of Technology

  Portions of this software were developed by the Graduate School of
  Oceanography (GSO) at the University of Rhode Island (URI) in collaboration
  with The Massachusetts Institute of Technology (MIT).

  Access and use of this software shall impose the following obligations and
  understandings on the user. The user is granted the right, without any fee
  or cost, to use, copy, modify, alter, enhance and distribute this software,
  and any derivative works thereof, and its supporting documentation for any
  purpose whatsoever, provided that this entire notice appears in all copies
  of the software, derivative works and supporting documentation.  Further,
  the user agrees to credit URI/MIT in any publications that result from the
  use of this software or in any product that includes this software. The
  names URI, MIT and/or GSO, however, may not be used in any advertising or
  publicity to endorse or promote any products or commercial entity unless
  specific written permission is obtained from URI/MIT. The user also
  understands that URI/MIT is not obligated to provide the user with any
  support, consulting, training or assistance of any kind with regard to the
  use, operation and performance of this software nor to provide the user
  with any updates, revisions, new versions or "bug fixes".

  THIS SOFTWARE IS PROVIDED BY URI/MIT "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL URI/MIT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS
  ACTION, ARISING OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE
  OF THIS SOFTWARE.
*/

// This is the main CGI program which extracts the requested variable from a
// netCDF data file. It generates the DDS in-memory from both the netCDF
// file and external file and uses it to locate the variable to process.
//
// ReZa 12/29/94

// $Log: nc_dods.cc,v $
// Revision 1.19  1999/01/21 01:42:25  jimg
// Added call to set filename in the DDS. This is required for proper operation
// of the projection functions.
//
// Revision 1.18  1998/03/19 23:23:11  jimg
// Fixed calls to set_mime_*().
//
// Revision 1.17  1997/12/30 23:53:07  jimg
// Fixed lame function declaration (which named the DDS object `das'). It
// bugged me...
//
// Revision 1.16  1997/08/27 00:14:43  jimg
// Changed main() to use the new DODSFilter class.
//
// Revision 1.15  1997/08/05 22:45:26  jimg
// Changed type of option_char in `option_char = getopt(...)' from char to
// int. This fixes a bug that shows up on the SGI (where char is unsigned).
//
// Revision 1.14  1997/06/05 17:21:06  jimg
// Rewrote main() so that it uses functions added to cgi_util.cc for the 2.14
// release. These function help to unify the *_dods filters so that they all
// respond more uniformly.
//
// Revision 1.13  1997/03/10 16:24:21  reza
// Added error object and upgraded to DODS core release 2.12.
//
// Revision 1.11  1997/02/07 17:58:20  reza
// Fixed so null constraint will return the entire data set now
//
// Revision 1.10  1996/11/01 00:18:33  jimg
// Added some limited error returns.
//
// Revision 1.9  1996/09/17 17:07:04  jimg
// Merge the release-2-0 tagged files (which were off on a branch) back into
// the trunk revision.
//
// Revision 1.8.2.2  1996/07/10 21:44:42  jimg
// Changes for version 2.06. These fixed lingering problems from the migration
// from version 1.x to version 2.x.
// Removed some (but not all) warning generated with gcc's -Wall option.
//
// Revision 1.8.2.1  1996/06/25 22:05:05  jimg
// Version 2.0 from Reza.
//
// Revision 1.8  1995/11/18  11:33:22  reza
// Updated member function names for DAP-1.1.1.
//
// Revision 1.7  1995/07/09  21:33:59  jimg
// Added copyright notice.
//
// Revision 1.6  1995/06/29  15:42:09  jimg
// Fixed nc_dods.cc
//
// Revision 1.5  1995/06/27  21:07:32  jimg
// Fixed up the error reporting. Added/corrected some comments.
//
// Revision 1.4  1995/06/23  15:50:35  jimg
// Changed from a CGI to a simple UNIX filter.
//
// Revision 1.3  1995/04/28  20:40:01  reza
// Moved hyperslab access to the server side (passed as a constraint argument).
//
// Revision 1.2  1995/03/16  16:44:06  reza
// Added TEST directives for data server debugging.
//
// Revision 1.1  1995/02/10  04:49:24  reza
// First version of Data Server for netCDF files.
//

static char rcsid[]={"$Id: nc_dods.cc,v 1.19 1999/01/21 01:42:25 jimg Exp $"};

#include <iostream.h>
#include <GetOpt.h>

#include "cgi_util.h"
#include "DODSFilter.h"
#include "DDS.h"

extern bool read_descriptors(DDS &dds, const char *filename, String *error);

int 
main(int argc, char *argv[])
{
    DDS dds;
    DODSFilter df(argc, argv);
	
    if (!df.OK()) {
	df.print_usage();
	return 1;
    }

    if (df.version()) {
	df.send_version_info();
	return 0;
    }

    dds.filename(df.get_dataset_name());

    // Read the netCDF file dataset descriptor in memory
    String errMsg;
    if(!read_descriptors(dds, df.get_dataset_name(), &errMsg)){
	ErrMsgT("nc_dods: could not read the DDS information");
	//client error message
	Error e(unknown_error, errMsg);
	set_mime_text(cout, dods_error);
	e.print(cout);
	return 1;
    }

    if (!df.read_ancillary_dds(dds))
	return 1;

    if (!df.send_data(dds, stdout))
	return 1;

    return 0;
}
